/*
 * Copyright (c) 2002-2003 Greg Kroah-Hartman <greg@kroah.com>
 * Copyright (c) 2002-2003 IBM Corp.
 *
 *	based on code from usbview which is Copyright (c) 1999, 2000 by
 *	Greg Kroah-Hartman, <greg@kroah.com>
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License as
 *	published by the Free Software Foundation; either version 2 of
 *	the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the Free
 *	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 *	USA.
 *	(See the included file COPYING)
 *
 */

#ifndef LOGGING_H
#define LOGGING_H

#include <stdio.h>
#include <syslog.h>


/* undef DO_LOGGING to turn off all logging */
#define DO_LOGGING

#ifdef DO_LOGGING
	#define dbg(format, arg...) do { log_message (LOG_DEBUG,    "%s: " format , __FUNCTION__ , ## arg); } while (0)
	#define info(format, arg...) do { log_message (LOG_INFO,    "%s: " format , __FUNCTION__ , ## arg); } while (0)
	#define warn(format, arg...) do { log_message (LOG_WARNING, "%s: " format , __FUNCTION__ , ## arg); } while (0)
	#define err(format, arg...) do { log_message (LOG_ERR,      "%s: " format , __FUNCTION__ , ## arg); } while (0)
	#define crit(format, arg...) do { log_message (LOG_CRIT,    "%s: " format , __FUNCTION__ , ## arg); } while (0)
#else
	#define dbg(format, arg...) do { } while (0)
	#define info(format, arg...) do { } while (0)
	#define warn(format, arg...) do { } while (0)
	#define err(format, arg...) do { } while (0)
	#define crit(format, arg...) do { } while (0)
#endif

/**
 * log_message - Sends a message to the logging facility
 *
 * It's recommended that you use the dbg, info, warn, macros yourself instead of calling this
 * function directly.
 * 
 * @level - the level this message is (LOG_DEBUG, LOG_INFO, etc.), from syslog.h
 * @fmt - a pointer to a string containing the message format.
 */
extern int log_message (int level, const char *fmt, ...)  __attribute__ ((__format__(__printf__, 2, 3)));

extern void log_init(void);
#endif
