/*
 * main.c for pcihpview - a pci hotplug viewing program
 *
 * Copyright (c) 2001-2003 Greg Kroah-Hartman <greg@kroah.com>
 * Copyright (c) 2001-2003 IBM Corp.
 *
 *	based on code from usbview which is Copyright (c) 1999, 2000 by
 *	Greg Kroah-Hartman, <greg@kroah.com>
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License as
 *	published by the Free Software Foundation; either version 2 of
 *	the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the Free
 *	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 *	USA.
 *	(See the included file COPYING)
 */

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/utsname.h>

#include <gtk/gtk.h>

#include "logging.h"
#include "interface.h"
#include "support.h"
#include "slot.h"

int get_kernel_version (int *major, int *minor, int *release)
{
	struct utsname utsname;
	int retval;
	char *temp, *temp2;

	retval = uname (&utsname);
	if (retval) {
		fprintf (stderr, "can't get current kernel version\n");
		return retval;
	}

	temp = strstr (utsname.release, ".");
	if (!temp)
		return -EINVAL;
	*temp = 0x00;
	*major = atoi(&utsname.release[0]);
	dbg ("major = %d\n", *major);
	
	temp++;
	temp2 = temp;
	temp = strstr (temp, ".");
	if (!temp)
		return -EINVAL;
	*temp = 0x00;
	*minor = atoi(temp2);
	dbg ("minor = %d\n", *minor);
	
	temp++;
	*release = atoi(temp);
	dbg ("release = %d\n", *release);

	return 0;
}

int main (int argc, char *argv[])
{
	GtkWidget *window_main;
	char *root_dir;
	int retval;

	log_init();

	if (argc <= 1) {
		int major, minor, release;
		int sysfs = 0;

		/* no command line override to where the directory it,
		 * let's try to guess where it should be */
		retval = get_kernel_version (&major, &minor, &release);
		if (retval)
			return retval;

		/* anything greater than 2.x.x should be using sysfs (hopefully...) */
		if (major > 2)
			sysfs = 1;

		/* anything over 2.4.x should also be using sysfs */
		if ((major == 2) && (minor > 4))
			sysfs = 1;

		if (sysfs) {
			/* default location of sysfs location */
			root_dir = "/sys/bus/pci/hotplug_slots/";
		} else {
			root_dir = "/proc/bus/pci/slots/";
		}
	} else {
		root_dir = argv[1];
	}
	dbg ("using location at %s\n", root_dir);

	gtk_set_locale ();
	gtk_init (&argc, &argv);

	window_main = create_main_window ();
	gtk_widget_show (window_main);

	retval = slot_set_mount (root_dir);
	if (retval) {
		fprintf (stderr, "error setting slot mount\n");
		return 1;
	}
	retval = slot_read_all ();
	if (retval) {
		fprintf (stderr, "error reading directory\n");
		return 1;
	}

	openlog (PACKAGE, 0, LOG_USER);
	dbg ("starting up at mount point %s\n", argv[1]);

	gtk_display_slots ();
	gtk_main ();

	dbg ("shutting down\n");
	closelog ();
	return 0;
}

